function [doutput, aoutput] = FRET_ck(donor, acceptor, WW, MM, PP)
    
% Counters to be used later for NaNs or Infs
d_NaNs_Infs = 0;  %counter for results that are NaN or +/- infinity
a_NaNs_Infs = 0;  %counter for results that are NaN or +/- infinity

% Determining the largest window size of all windows, which dictates how close to the ends of the data you can smooth.
Wmax = max(WW);
comp = [MM Wmax];
biggest = max(comp)+1;

% Finding the lengths.
lenW = length(WW); lenP = length(PP); lenD = length(donor); lenA = length(acceptor);

if lenD > lenA
     lendata = lenA;
else lendata = lenD;
end

% Output matrices.
fwpred(lendata, lenW) = 0;  bapred(lendata, lenW) = 0;  % predictors
dfwmean(lendata, lenW) = 0; abamean(lendata, lenW) = 0; % forward means
afwmean(lendata, lenW) = 0; dbamean(lendata, lenW) = 0; % backward means

% Calculating Predictors and Means in the various windows of data before and after each timepoint.

% Looping through the timepoints
for tpts = 1:lendata;
    
    % Looping through the predictor window sizes
    for wdws = 1:lenW,    W = WW(wdws);
        
        % If we're at a data point within the largest window size from the start or end of the data...
        if tpts-biggest-1 >0 && tpts+biggest+1<= lendata
            
           bapred(tpts,wdws) = std(donor(tpts-MM-1:tpts-1))^2 + std(acceptor(tpts-MM-1:tpts-1))^2;
           fwpred(tpts,wdws) = std(donor(tpts+1:tpts+MM+1))^2 + std(acceptor(tpts+1:tpts+MM+1))^2;
           
           dbamean(tpts,wdws) = mean(donor(tpts-W-1:tpts-1));       % donor backward mean
           abamean(tpts,wdws) = mean(acceptor(tpts-W-1:tpts-1));    % acceptor backward mean
 
           dfwmean(tpts,wdws) = mean(donor(tpts+1:tpts+W+1));    % donor forward mean
           afwmean(tpts,wdws) = mean(acceptor(tpts+1:tpts+W+1)); % acceptor forward mean
        end
    end
end 

            
% Weighting each Mean

% Looping through the timepoints
for tpts = 1:lendata;
    
    % Looping through the exponents
    for exps = 1:lenP
        
        % Output matrices.
        dfw = zeros(lendata, lenP); dba = zeros(lendata, lenP);
        afw = zeros(lendata, lenP); aba = zeros(lendata, lenP);
        fss = zeros(lendata, lenP); bss = zeros(lendata, lenP);

        % Looping through the predictor window sizes
        for wdws = 1:lenW
            
            dba(tpts,exps) = dba(tpts,exps) + ((bapred(tpts,wdws))^(-2*PP(exps)))*dbamean(tpts,wdws);
            aba(tpts,exps) = aba(tpts,exps) + ((bapred(tpts,wdws))^(-2*PP(exps)))*abamean(tpts,wdws);
            afw(tpts,exps) = afw(tpts,exps) + ((fwpred(tpts,wdws))^(-2*PP(exps)))*afwmean(tpts,wdws);
            dfw(tpts,exps) = dfw(tpts,exps) + ((fwpred(tpts,wdws))^(-2*PP(exps)))*dfwmean(tpts,wdws);
            
            bss(tpts,exps) = bss(tpts,exps) + ((bapred(tpts,wdws))^(-2*PP(exps)));
            fss(tpts,exps) = fss(tpts,exps) + ((fwpred(tpts,wdws))^(-2*PP(exps)));            

            % Does the donor signal contain NaNs or Infs?
            dnan =  (find(isnan([dfw(tpts,exps) dba(tpts,exps) fss(tpts,exps) bss(tpts,exps) fwpred(tpts,exps) bapred(tpts,exps) dfwmean(tpts,exps) dbamean(tpts,exps)])>0));
            dinf =  (find(isinf([dfw(tpts,exps) dba(tpts,exps) fss(tpts,exps) bss(tpts,exps) fwpred(tpts,exps) bapred(tpts,exps) dfwmean(tpts,exps) dbamean(tpts,exps)])>0));

            if  tpts+W+1 <= lendata && tpts-W-1 > 0 && (length(dnan) >= 1 || (length(dinf)) >= 1)
                d_NaNs_Infs = d_NaNs_Infs + 1;    
                dNanInf_index(d_NaNs_Infs,:) = [exps,tpts,wdws];
            end

            % Does the acceptor signal contain NaNs or Infs?
            anan =  (find(isnan([afw(tpts,exps) aba(tpts,exps) fss(tpts,exps) bss(tpts,exps) fwpred(tpts,exps) bapred(tpts,exps) afwmean(tpts,exps) abamean(tpts,exps)])>0));
            ainf =  (find(isinf([afw(tpts,exps) aba(tpts,exps) fss(tpts,exps) bss(tpts,exps) fwpred(tpts,exps) bapred(tpts,exps) afwmean(tpts,exps) abamean(tpts,exps)])>0));

            if  tpts+W+1<= lendata && tpts-W-1 >0 && (length(anan) >= 1 || (length(ainf)) >= 1)
                a_NaNs_Infs = a_NaNs_Infs + 1;
                aNanInf_index(a_NaNs_Infs ,:) = [exps,tpts,wdws];
            end
        end

        % Calculating the smoothed data.
        if tpts+W+1<= lendata && tpts-W-1 >0
            dsmooth(tpts,exps) =  (dfw(tpts,exps) + dba(tpts,exps))/(bss(tpts,exps) + fss(tpts,exps));
            asmooth(tpts,exps) =  (afw(tpts,exps) + aba(tpts,exps))/(bss(tpts,exps) + fss(tpts,exps));
        end
    end 
end

% Fixing up smoothed donor data NaNs and Infs
doutput{1} = dsmooth;
dmod = dsmooth;
dfinal = dsmooth;

if d_NaNs_Infs > 0
    dcorrections(length(dsmooth)) = 0;
end 

for dNaNInfctr = 1:d_NaNs_Infs 
    exps = dNanInf_index(dNaNInfctr,1);
    tpts = dNanInf_index(dNaNInfctr,2);
    wdws = dNanInf_index(dNaNInfctr,3);

    good_d_before = find(dsmooth(1:tpts-1,exps) > 0 & (dsmooth(1:tpts-1,exps) < 1), 2, 'last' )';
    good_d_after = find(dsmooth(tpts+1:lendata-biggest,exps) > 0 & (dsmooth(tpts+1:lendata-biggest,exps) < 1), 2, 'first')';
    dmod(tpts,exps) = mean([(dsmooth(good_d_before,exps))' (dsmooth(tpts+good_d_after,exps))']);
    dcorrections(tpts) = dmod(tpts,exps);
end

if d_NaNs_Infs > 0
    dg = find(dcorrections ~= 0);
    dfinal(dg) = dcorrections(dg);
end
   
if d_NaNs_Infs > 0
    doutput{2} = dmod;
else
    doutput{2} = dsmooth;
end
doutput{3} = dfinal;

% Fixing up smoothed acceptor data NaNs and Infs
aoutput{1} = asmooth;
amod = asmooth;
afinal = asmooth;

if a_NaNs_Infs > 0
    acorrections(length(asmooth)) = 0;
end

for aNaNInfctr = 1:a_NaNs_Infs 
    exps = aNanInf_index(aNaNInfctr,1);
    tpts = aNanInf_index(aNaNInfctr,2);
    wdws = aNanInf_index(aNaNInfctr,3);

    good_a_before = find(asmooth(1:tpts-1,exps) >0 & (asmooth(1:tpts-1,exps) <1), 2, 'last' )';
    good_a_after = find(asmooth(tpts+1:lendata-biggest,exps) >0 & (asmooth(tpts+1:lendata-biggest,exps) <1), 2, 'first')';
    amod(tpts,exps) = mean([(asmooth(good_a_before,exps))' (asmooth(tpts+good_a_after,exps))']);
    acorrections(tpts) = amod(tpts,exps);
end 

if a_NaNs_Infs > 0
    ag = find(acorrections ~= 0);
    afinal(ag) = acorrections(ag);
end
   
if a_NaNs_Infs > 0
    aoutput{2} = amod;
else
    aoutput{2} = asmooth;
end
aoutput{3} = afinal;
