function [baddata] = baddata(donorraw, acceptorraw, donorsmooth, acceptorsmooth)

% Determining the 95% confidence limits of the raw data using the standard deviation of the points near each timepoint. 
len=min(length(donorraw),length(acceptorraw));
CIwindow=2; dCI = zeros(2,CIwindow); aCI = zeros(2,CIwindow);
for ctr = 1+CIwindow:len-CIwindow
    dwindow = donorraw(ctr-CIwindow:ctr+CIwindow);
    awindow = acceptorraw(ctr-CIwindow:ctr+CIwindow);
    dSEM = std(dwindow)/sqrt(2*CIwindow+1);
    aSEM = std(awindow)/sqrt(2*CIwindow+1);
    ts = tinv([0.025 0.975], 2*CIwindow);
    dCI(:,ctr) = mean(dwindow) + ts*dSEM;
    aCI(:,ctr) = mean(awindow) + ts*aSEM;
end

% Smoothing the confidence limits using the Chung-Kennedy filter.
dCI1filterme = dCI(1,:); dCI1filterme = [dCI1filterme zeros(1,CIwindow)]; 
dCI2filterme = dCI(2,:); dCI2filterme = [dCI2filterme zeros(1,CIwindow)];
[dCI1, dCI2] = FRET_ck(dCI1filterme, dCI2filterme, [2,10], 5, 2);
dCIsmooth(1,:) = dCI1{3};
dCIsmooth(2,:) = dCI2{3}; 
            
aCI1filterme = aCI(1,:); aCI1filterme = [aCI1filterme zeros(1,CIwindow)];    
aCI2filterme = aCI(2,:); aCI2filterme = [aCI2filterme zeros(1,CIwindow)];
[aCI1, aCI2] = FRET_ck(aCI1filterme, aCI2filterme, [2,10], 5, 2);
aCIsmooth(1,:) = aCI1{3};
aCIsmooth(2,:) = aCI2{3};
    
% Making blmarker, an array that is 1 (true) when the confidence interval for a timepoint contains zero. 
dblmarker = zeros(1,length(dCIsmooth)); ablmarker = zeros(1,length(aCIsmooth)); blmarker = zeros(1,length(aCIsmooth)); 
for blctr = 1:length(dCIsmooth)
    if dCIsmooth(1,blctr) <=0 && dCIsmooth(2,blctr) >=0
        dblmarker(blctr) = 1;
    end
    if aCIsmooth(1,blctr) <=0 && aCIsmooth(2,blctr) >=0
        ablmarker(blctr) = 1;
    end
    if dblmarker(blctr) == 1 || ablmarker(blctr) == 1
        blmarker(blctr) = 1;
    end
end

% Adding in times were the donor or acceptor intensity are <= 100 or are >=3000 into blmarker.
for blctr2 = 1+CIwindow:length(blmarker)-CIwindow
    if donorsmooth(blctr2) <= 100 || acceptorsmooth(blctr2) <= 100
        blmarker(blctr2) = 1;
    end
    if donorsmooth(blctr2) >= 10000 || acceptorsmooth(blctr2) >= 10000
        blmarker(blctr2) = 1;
    end
end

% Removing shortlived states identified as on or off.
blwindow=9; iteration=1;
blmarker(iteration+1,:) = blmarker(iteration,:); 
for blctr_1 = 1+blwindow:length(blmarker)-blwindow
    for i = 0:blwindow
        if blmarker(iteration,blctr_1-i) == 1
            for ii = 0:blwindow
                if blmarker(iteration,blctr_1+i) == 1
                    blmarker(iteration+1,blctr_1) = 1;
                end
            end
        elseif blmarker(iteration,blctr_1-i) ==0
            for ii = 0:blwindow
                if blmarker(iteration,blctr_1+i) == 0
                    blmarker(iteration+1,blctr_1) = 0;
                end
            end                        
        end
    end
end             
while isequal(blmarker(iteration,:),blmarker(iteration+1,:)) == 0             
    iteration = iteration + 1;
    blmarker(iteration+1,:) = blmarker(iteration,:);                
    for blctr_1 = 1+blwindow:length(blmarker)-blwindow
        for i = 0:blwindow
            if blmarker(iteration,blctr_1-i) == 1
                for ii = 0:blwindow
                    if blmarker(iteration,blctr_1+i) == 1
                        blmarker(iteration+1,blctr_1) = 1;
                    end
                end
            elseif blmarker(iteration,blctr_1-i) ==0
                for ii = 0:blwindow
                    if blmarker(iteration,blctr_1+i) == 0
                        blmarker(iteration+1,blctr_1) = 0;
                    end
                end
            end
        end
    end
end

baddata(1,:) = blmarker(end,:);
baddata(1,end) = 1;

%Pengning add this filter on 10/16/2017, sometimes the
%trace pass filter because last few timepoints are regarded as
%good data. So I change the data manually.
 for i = len-15 : len
     baddata(1,i) = 1;
 end