function [GKtransitions, CKtransitions, output] = align_extract_trans_data(donor, acceptor, fret, GKtransindex, CKtransindex, GKtransitions, CKtransitions, starttime, endtime, timestep)

time = 1:1:length(fret);

if isempty(GKtransindex) == 1 && isempty(CKtransindex) == 1
    region = 1:endtime/timestep;
    stats(1,:) = [nanmean(fret(region)), nanstd(fret(region))];
    stats2(1,:) = [nanmean(donor(region)),  nanmean(acceptor(region)), nanstd(donor(region)), nanstd(acceptor(region))];
    stats3(1,:) = [nanmean(fret(region)) + tinv([0.025 0.975], length(region))*(nanstd(fret(region))/sqrt(2*length(region)+1)), ...
        nanmean(donor(region)) + tinv([0.025 0.975], length(region))*(nanstd(donor(region))/sqrt(2*length(region)+1)), ...
        nanmean(acceptor(region)) + tinv([0.025 0.975], length(region))*(nanstd(acceptor(region))/sqrt(2*length(region)+1))];
    
    output = zeros(length(fret),12);
    for t = starttime/timestep-1:endtime/timestep
        output(t,:) = [stats(1,1), stats(1,2), stats2(1,:), stats3(1,:)];
    end
else
    
    if isempty(GKtransindex) == 1 && isempty(CKtransindex) ~= 1
        uniquetrans = unique(CKtransindex);
    elseif isempty(GKtransindex) ~= 1 && isempty(CKtransindex) == 1
        uniquetrans = unique(GKtransindex);
    else
        % Aligning transitions detected two ways.
        lenGK = length(GKtransindex);
        CKfudge = [0]; GKfudge = [0];
        for index = 1:lenGK
            for wiggle = 0:6
                [a,~,b] = find(CKtransindex == (GKtransindex(index)+wiggle-3));
                CKfudge = [CKfudge a];
                if b == 1
                    GKfudge = [GKfudge index];
                end
            end
        end

        CKfudge(CKfudge==0)=[]; GKfudge(GKfudge==0)=[];

        for fdg = 1:length(CKfudge)
            CKpicks = CKtransitions(CKtransindex(CKfudge(fdg)));
            GKpicks = GKtransitions(GKtransindex(GKfudge(fdg)));
            CKtransitions(CKtransindex(CKfudge(fdg)))=0;
            GKtransitions(GKtransindex(GKfudge(fdg)))=0;
            wgtCK = CKpicks*CKtransindex(CKfudge(fdg));
            wgtGK = GKpicks*GKtransindex(GKfudge(fdg));
            CKtransindex(CKfudge(fdg)) = round((wgtCK + wgtGK)/(CKpicks+GKpicks));
            GKtransindex(GKfudge(fdg)) = CKtransindex(CKfudge(fdg));
            CKtransitions(CKtransindex(CKfudge(fdg)))=CKpicks;
            GKtransitions(GKtransindex(GKfudge(fdg)))=GKpicks;
        end

        alltransindex = [GKtransindex; CKtransindex];
        uniquetrans = unique(alltransindex);
    end

    % Once transitions are found, pull information from the data.
    output = zeros(length(fret),12);


        % First Transition:
    region = 1:uniquetrans(1);
    stats(1,:) = [nanmean(fret(region)), nanstd(fret(region))];
    stats2(1,:) = [nanmean(donor(region)),  nanmean(acceptor(region)), nanstd(donor(region)), nanstd(acceptor(region))];
    stats3(1,:) = [nanmean(fret(region)) + tinv([0.025 0.975], length(region))*(nanstd(fret(region))/sqrt(2*length(region)+1)), ...
        nanmean(donor(region)) + tinv([0.025 0.975], length(region))*(nanstd(donor(region))/sqrt(2*length(region)+1)), ...
        nanmean(acceptor(region)) + tinv([0.025 0.975], length(region))*(nanstd(acceptor(region))/sqrt(2*length(region)+1))];
        % Middle Transitions
    if length(uniquetrans) > 1
        for k = 2:length(uniquetrans)
            region = uniquetrans(k-1):uniquetrans(k);
            stats(k,:) = [nanmean(fret(region)), nanstd(fret(region))];
            stats2(k,:) = [nanmean(donor(region)), nanmean(acceptor(region)), nanstd(donor(region)) nanstd(acceptor(region))];
            stats3(k,:) = [nanmean(fret(region)) + tinv([0.025 0.975], length(region))*(nanstd(fret(region))/sqrt(2*length(region)+1)), ...
                nanmean(donor(region)) + tinv([0.025 0.975], length(region))*(nanstd(donor(region))/sqrt(2*length(region)+1)), ...
                nanmean(acceptor(region)) + tinv([0.025 0.975], length(region))*(nanstd(acceptor(region))/sqrt(2*length(region)+1))];
        end
    end
        % Last Transition
    region = uniquetrans(length(uniquetrans)):endtime/timestep;
    stats(end+1,:) = [nanmean(fret(region)), nanstd(fret(region))];
    stats2(end+1,:) = [nanmean(donor(region)), nanmean(acceptor(region)), nanstd(donor(region)) nanstd(acceptor(region))];
    stats3(end+1,:) = [nanmean(fret(region)) + tinv([0.025 0.975], length(region))*(nanstd(fret(region))/sqrt(2*length(region)+1)), ...
        nanmean(donor(region)) + tinv([0.025 0.975], length(region))*(nanstd(donor(region))/sqrt(2*length(region)+1)), ...
        nanmean(acceptor(region)) + tinv([0.025 0.975], length(region))*(nanstd(acceptor(region))/sqrt(2*length(region)+1))];

        % Start writing an output matrix
    for state = 1:length(uniquetrans)+1
        if state == 1
            for t = starttime/timestep-1:time(uniquetrans(state)-1)
                output(t,:) = [stats(state,1), stats(state,2), stats2(state,:), stats3(state,:)];
            end
        elseif state == length(uniquetrans)+1
            for t = time(uniquetrans(state-1)):endtime/timestep
                output(t,:) = [stats(state,1), stats(state,2), stats2(state,:), stats3(state,:)];
            end
        else
            for t = time(uniquetrans(state-1)):time(uniquetrans(state))-1
                output(t,:) = [stats(state,1), stats(state,2), stats2(state,:), stats3(state,:)];
            end
        end
    end
end