function [databad,CKtrans_aligned,fret_avg] = Edgefind(raw,result3,starttime,endtime,timestep)
result0 = {'N','N','Y'};
result1={'2 4 8','4','10'};
WW = str2num(result1{1});        % window size(s) for predictors
MM = str2num(result1{2});        % window for comparison
PP = str2num(result1{3});        % exponent for making the forward and backward predictors
result2={'0.2 0.3 0.4 0.5 0.6','1 2 4 8','2'};

Ntraces=2;
len = length(raw);
% Setting up matrices to be used later.
time1 = zeros(1,len);
Data=zeros(Ntraces+1,len);
donor=zeros(Ntraces/2,len);
acceptor=zeros(Ntraces/2,len);
time_1 = zeros(len,1);
donor_1 = zeros(len,Ntraces/2);
acceptor_1 = zeros(len,Ntraces/2);
donorraw = zeros(len,Ntraces/2);
acceptorraw = zeros(len,Ntraces/2);
fret_smth = zeros(len,Ntraces/2);
fretraw = zeros(len,Ntraces/2);
GKtransitions = zeros(len,Ntraces/2); GKtransindex = [];
CKtransitions = zeros(len,Ntraces/2); CKtransindex = [];



% Counters used to go forward and backward when sorting
molnumber=1;


% Looping through all molecules
while molnumber < (Ntraces/2 + 1) 
    % Applying correction factors
    time_1 = raw(:,1);
    donor_1 = raw(:,2);                    % Raw donor signal is corrected with the gamma factor
    acceptor_1 = raw(:,3);	% Raw acceptor signal is corrected with the leakage factor
    donorraw=donor_1;
    acceptorraw=acceptor_1;

    % Applying the Chung-Kennedy filter            
    if result0{1} == 'Y'
        disp(['      Smoothing the data using the Chung-Kennedy filter.'])
        dfilterme = donor_1(:,1);     
        afilterme = acceptor_1(:,1);
        donorraw=donor_1(:,1);
        acceptorraw=acceptor_1(:,1);
        [doutput, aoutput] = FRET_ck(dfilterme, afilterme, WW, MM, PP);
        donor_1 = doutput{3};
        acceptor_1 = aoutput{3};
    else
        donorraw = donor_1(:,1);
        acceptorraw = acceptor_1(:,1);
    end

    % Calculating FRET.
    if length(acceptor_1) > length(donor_1)
        len = length(donor_1);
    else
        len = length(acceptor_1);
    end

    % Identifying bleaches, blinks, and intensities that are too high or too low.
    disp(['      Checking for bad data.'])
    databad = 0;
    databad = baddata(donorraw, acceptorraw, donor_1, acceptor_1);
    databad = zeros(1,len);


    % If the whole molecule is bad, skip to the next one.
    if isequal(databad,ones(1,len))==1
        disp(['      * * * Molecule ' num2str(molnumber) ' was entirely unusable. * * *'])
        molnumber=molnumber+1;

    % Ensure 0 < FRET < 1 and make FRET = 0 at bleaches and blinks.
    else
        fretres = zeros(1,len);
        disp(['      Calculating FRET.'])                   
        for t = 1:len
            ft(t)=0;
            if databad(t) ==1
                fret_smth(t) = 0;
                fretraw(t) = 0;
            else
                fret_smth(t) = acceptor_1(t)/(donor_1(t)+acceptor_1(t));
                fretraw(t) = acceptorraw(t)/(donorraw(t)+acceptorraw(t));
                fretres(t)= fretraw(t)-fret_smth(t);
                if  fret_smth(t) > 0.9999
                    fret_smth(t) = 0.9999;
                elseif fret_smth(t) < 0
                    fret_smth(t) = 0.0001;
                end
            end
        end

        % Making a histogram of FRET at each timepoint, removing zeros and ones.
        histogram=0; xout=0; count=0; hist_ctr=0;         
        for tt = 1:len
            if fret_smth(tt)<.05 || fret_smth(tt)>.95
                count=count+1;
            else
                hist_ctr=tt-count;
                ft(hist_ctr)=fret_smth(tt);          
            end
        end
        frt=[0 1];
        for hh=1:hist_ctr
            frt(hh+2)=ft(hh);
        end

        histogram=[hist(frt,50) zeros(1,len-50)];

        % Detecting edges using the Gaussian kernel.
        if result0{2} == 'Y'
            disp(['      Detecting edges using the Gaussian kernel.'])
            thresholds = str2num(result2{1});
            scales = str2num(result2{2});
            tranrad = str2num(result2{3});
            [GKtransitions, GKtransindex] = GKedgedetection(donor_1, acceptor_1, fret_smth, tranrad, starttime, endtime, timestep, thresholds, deriv, scales, width);
        end

        % Detecting edges using the Chung-Kennedy filter.
        if result0{3} == 'Y'
            disp(['      Detecting edges using the Chung-Kennedy filter.'])
            CKW = str2num(result3{1});
            PCT = str2num(result3{2});
            SEP = str2num(result3{3});
            if mod(CKW,2) == 1
                CKW = CKW+1;
            end
            [CKtransitions, CKtransindex] = CKedgedetection(donor_1, acceptor_1, fret_smth, CKW, PCT, SEP, starttime, timestep);
        end


        % Aligning the results of the two edge detection methods weighted by how many times that method detected the transition.
        disp(['      Aligning the results of the edge detection methods.'])
        endtp = str2num(num2str(len*timestep));
        [GKtrans_aligned, CKtrans_aligned, output] = align_extract_trans_data(donor_1, acceptor_1, fret_smth, GKtransindex, CKtransindex, GKtransitions, CKtransitions, starttime, endtp, timestep);

        fret_avg = output(:,1);

        if length(fret_smth) < length(donor_1)
            lent = length(fret_smth);
        else lent = length(acceptor_1);
        end

        molnumber=molnumber+1; 
    end
end