function [GKtransitions, GKtransindex] = GKedgedetection(donorin, acceptorin, fretin, tranrad, starttime, endtime, timestep, thresholds, deriv, scales, width)

% Read in donor and acceptor (smoothed), FRET, and timesteps.
donor = donorin((starttime/timestep)+1:(endtime/timestep)+1);
acceptor = acceptorin((starttime/timestep)+1:(endtime/timestep)+1);
data = fretin((starttime/timestep)+1:(endtime/timestep)+1); 
time = starttime:timestep:(length(data)-1)*timestep + starttime; 

% Calculating Gaussian Kernel Parameters
for k = 1:length(scales)
    sigma = scales(k);
    range = 1:2*width*(ceil(sigma)) + 1;
    center = range(ceil(length(range)/2));
    derivs(1,1:length(range)) = 1;
    derivs(2,1:length(range)) = -((range-center)/(sigma^2));
    derivs(3,1:length(range)) = ((range-center).^2 - (sigma^2))/(sigma^4);
    derivs(4,1:length(range)) = -((range-center).^3 - 3 * (sigma^2) * (range-center))/(sigma^6);
    derivs(5,1:length(range)) = ((range-center).^4 - 6 * (sigma^2) *        (range-center).^2 + 3 * (sigma^4))/(sigma^8);
    kernel = (1/(sigma*sqrt(2*pi))) * exp(-((range-center).^2)/(2*sigma^2));
    g = kernel .* derivs(deriv+1,:);
    padData = [data(1)*ones(length(g),1); data'; data(end)*ones(length(g),1)];
    fData = conv(padData, g);
    offset = (length(fData) - length(data)) / 2;
    dData = fData(offset:offset+length(data)-1);
end;

numscales = scales(size(scales,2));
lookdata = dData(:,(size(dData,2)));
rdataMax = lookdata /0.01;
rdataMin = lookdata /(-0.01);

winmax = zeros(size(lookdata));
winmin = zeros(size(lookdata));

% Note: In this for loop, sigma is from the largest scale (the last scale of the previous for loop).
for k = 1+sigma:size(lookdata,1)-sigma 
    winmax(k) = max(rdataMax(k-sigma:k+sigma));
    winmin(k) = max(rdataMin(k-sigma:k+sigma));
end

% Searching for maxima and minima in the kernel that indicate transitions.
maxima(size(lookdata,1), length(thresholds)) = 0;
minima(size(lookdata,1), length(thresholds)) = 0;

for kk = 1+numscales:(size(lookdata,1)-numscales);
    for kkk = 1:length(thresholds)
        if (rdataMax(kk) >= thresholds(kkk)) & (rdataMax(kk) >= winmax(kk));
            maxima(kk,kkk)  = 1;
        end
        if (rdataMin(kk) >= thresholds(kkk)) & (rdataMin(kk) >= winmin(kk));
            minima(kk,kkk) = 1;
        end
    end
end

% Applying a t-test to the FRET states between each transition, and only keeping significantly different states.
transitions = maxima + minima;
indeces = find(transitions > 1); transitions(indeces) = 1;
transitions = [zeros((starttime/timestep)-1,length(thresholds)); transitions; zeros((length(fretin)-length(transitions)-(starttime/timestep)+1),length(thresholds))];
GKtransitions = sum(transitions,2);

boundaries = 0; alltransindex = 0;
alltransindex = find(GKtransitions(:,1)); 
boundaries = [0; alltransindex; length(data)];
if max(boundaries) ~= boundaries(end)
    boundaries(boundaries==max(boundaries))=[];
end
ttestctr = 1;

while ttestctr < (length(boundaries)-2)
    testregion = boundaries(ttestctr)+tranrad:boundaries(ttestctr+1)-tranrad;
    testnextregion = boundaries(ttestctr+1)+tranrad:boundaries(ttestctr+2)-tranrad;
    ttestresult = ttest2(mean(data(testregion)), mean(data(testnextregion)), [], [], 'unequal');
    if ttestresult == 1
        ttestctr = ttestctr+1;
    else
        boundaries(ttestctr+1) = [];
    end
end
   
GKtransindex(:,1) = boundaries(2:end-1);
badtransindex = (1:1:length(data));
for a = 1:length(GKtransindex)
    badtransindex(GKtransindex(a))=0;
    a=a+1;
end
badtransindex(badtransindex==0)=[];
for b = 1:length(badtransindex)
    GKtransitions(badtransindex(b))=0;
    b=b+1;
end